@@ -217,7 +217,7 @@ tp_new_touch(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
 	t->was_down = false;
 	t->state = TOUCH_HOVERING;
 	t->pinned.is_pinned = false;
-	t->millis = time;
+	t->time = time;
 	tp->queued |= TOUCHPAD_EVENT_MOTION;
 }
 
@@ -226,7 +226,7 @@ tp_begin_touch(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
 {
 	t->dirty = true;
 	t->state = TOUCH_BEGIN;
-	t->millis = time;
+	t->time = time;
 	t->was_down = true;
 	tp->nfingers_down++;
 	t->palm.time = time;
@@ -259,7 +259,7 @@ tp_end_touch(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
 	t->palm.state = PALM_NONE;
 	t->state = TOUCH_END;
 	t->pinned.is_pinned = false;
-	t->millis = time;
+	t->time = time;
 	t->palm.time = 0;
 	assert(tp->nfingers_down >= 1);
 	tp->nfingers_down--;
@@ -276,6 +276,14 @@ tp_end_sequence(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
 	tp_end_touch(tp, t, time);
 }
 
+static void
+tp_stop_actions(struct tp_dispatch *tp, uint64_t time)
+{
+	tp_edge_scroll_stop_events(tp, time);
+	tp_gesture_cancel(tp, time);
+	tp_tap_suspend(tp, time);
+}
+
 struct normalized_coords
 tp_get_delta(struct tp_touch *t)
 {
@@ -306,7 +314,7 @@ tp_process_absolute(struct tp_dispatch *tp,
 						  e->code,
 						  e->value);
 		t->point.x = e->value;
-		t->millis = time;
+		t->time = time;
 		t->dirty = true;
 		tp->queued |= TOUCHPAD_EVENT_MOTION;
 		break;
@@ -315,7 +323,7 @@ tp_process_absolute(struct tp_dispatch *tp,
 						  e->code,
 						  e->value);
 		t->point.y = e->value;
-		t->millis = time;
+		t->time = time;
 		t->dirty = true;
 		tp->queued |= TOUCHPAD_EVENT_MOTION;
 		break;
@@ -330,6 +338,13 @@ tp_process_absolute(struct tp_dispatch *tp,
 		break;
 	case ABS_MT_PRESSURE:
 		t->pressure = e->value;
+		t->time = time;
+		t->dirty = true;
+		tp->queued |= TOUCHPAD_EVENT_OTHERAXIS;
+		break;
+	case ABS_MT_TOOL_TYPE:
+		t->is_tool_palm = e->value == MT_TOOL_PALM;
+		t->time = time;
 		t->dirty = true;
 		tp->queued |= TOUCHPAD_EVENT_OTHERAXIS;
 		break;
@@ -349,7 +364,7 @@ tp_process_absolute_st(struct tp_dispatch *tp,
 						  e->code,
 						  e->value);
 		t->point.x = e->value;
-		t->millis = time;
+		t->time = time;
 		t->dirty = true;
 		tp->queued |= TOUCHPAD_EVENT_MOTION;
 		break;
@@ -358,12 +373,13 @@ tp_process_absolute_st(struct tp_dispatch *tp,
 						  e->code,
 						  e->value);
 		t->point.y = e->value;
-		t->millis = time;
+		t->time = time;
 		t->dirty = true;
 		tp->queued |= TOUCHPAD_EVENT_MOTION;
 		break;
 	case ABS_PRESSURE:
 		t->pressure = e->value;
+		t->time = time;
 		t->dirty = true;
 		tp->queued |= TOUCHPAD_EVENT_OTHERAXIS;
 		break;
@@ -607,6 +623,31 @@ tp_palm_detect_trackpoint_triggered(struct tp_dispatch *tp,
 	return false;
 }
 
+static bool
+tp_palm_detect_tool_triggered(struct tp_dispatch *tp,
+			      struct tp_touch *t,
+			      uint64_t time)
+{
+	if (!tp->palm.use_mt_tool)
+		return false;
+
+	if (t->palm.state != PALM_NONE &&
+	    t->palm.state != PALM_TOOL_PALM)
+		return false;
+
+	if (t->palm.state == PALM_NONE &&
+	    t->is_tool_palm)
+		t->palm.state = PALM_TOOL_PALM;
+	else if (t->palm.state == PALM_TOOL_PALM &&
+		 !t->is_tool_palm)
+		t->palm.state = PALM_NONE;
+
+	if (t->palm.state == PALM_TOOL_PALM)
+		tp_stop_actions(tp, time);
+
+	return t->palm.state == PALM_TOOL_PALM;
+}
+
 static inline bool
 tp_palm_detect_move_out_of_edge(struct tp_dispatch *tp,
 				struct tp_touch *t,
@@ -657,16 +698,11 @@ tp_palm_detect_multifinger(struct tp_dispatch *tp, struct tp_touch *t, uint64_t
 	return false;
 }
 
-static void
-tp_palm_detect(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
+static inline bool
+tp_palm_detect_edge(struct tp_dispatch *tp,
+		    struct tp_touch *t,
+		    uint64_t time)
 {
-
-	if (tp_palm_detect_dwt_triggered(tp, t, time))
-		goto out;
-
-	if (tp_palm_detect_trackpoint_triggered(tp, t, time))
-		goto out;
-
 	if (t->palm.state == PALM_EDGE) {
 		if (tp_palm_detect_multifinger(tp, t, time)) {
 			t->palm.state = PALM_NONE;
@@ -682,36 +718,78 @@ tp_palm_detect(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
 			evdev_log_debug(tp->device,
 				  "palm: touch released, out of edge zone\n");
 		}
-		return;
+		return false;
 	} else if (tp_palm_detect_multifinger(tp, t, time)) {
-		return;
+		return false;
 	}
 
 	/* palm must start in exclusion zone, it's ok to move into
 	   the zone without being a palm */
 	if (t->state != TOUCH_BEGIN ||
 	    (t->point.x > tp->palm.left_edge && t->point.x < tp->palm.right_edge))
-		return;
+		return false;
 
 	/* don't detect palm in software button areas, it's
 	   likely that legitimate touches start in the area
 	   covered by the exclusion zone */
 	if (tp->buttons.is_clickpad &&
 	    tp_button_is_inside_softbutton_area(tp, t))
-		return;
+		return false;
 
 	if (tp_touch_get_edge(tp, t) & EDGE_RIGHT)
-		return;
+		return false;
 
 	t->palm.state = PALM_EDGE;
 	t->palm.time = time;
 	t->palm.first = t->point;
 
+	return true;
+}
+
+static void
+tp_palm_detect(struct tp_dispatch *tp, struct tp_touch *t, uint64_t time)
+{
+	const char *palm_state;
+	enum touch_palm_state oldstate = t->palm.state;
+
+	if (tp_palm_detect_dwt_triggered(tp, t, time))
+		goto out;
+
+	if (tp_palm_detect_trackpoint_triggered(tp, t, time))
+		goto out;
+
+	if (tp_palm_detect_tool_triggered(tp, t, time))
+		goto out;
+
+	if (tp_palm_detect_edge(tp, t, time))
+		goto out;
+
+	return;
 out:
+	if (oldstate == t->palm.state)
+		return;
+
+	switch (t->palm.state) {
+	case PALM_EDGE:
+		palm_state = "edge";
+		break;
+	case PALM_TYPING:
+		palm_state = "typing";
+		break;
+	case PALM_TRACKPOINT:
+		palm_state = "trackpoint";
+		break;
+	case PALM_TOOL_PALM:
+		palm_state = "tool-palm";
+		break;
+	case PALM_NONE:
+	default:
+		abort();
+		break;
+	}
 	evdev_log_debug(tp->device,
 		  "palm: palm detected (%s)\n",
-		  t->palm.state == PALM_EDGE ? "edge" :
-		  t->palm.state == PALM_TYPING ? "typing" : "trackpoint");
+		  palm_state);
 }
 
 static inline const char*
@@ -810,13 +888,18 @@ tp_unhover_pressure(struct tp_dispatch *tp, uint64_t time)
 		if (t->dirty) {
 			if (t->state == TOUCH_HOVERING) {
 				if (t->pressure >= tp->pressure.high) {
+					evdev_log_debug(tp->device,
+							"pressure: begin touch\n");
 					/* avoid jumps when landing a finger */
 					tp_motion_history_reset(t);
 					tp_begin_touch(tp, t, time);
 				}
 			} else {
-				if (t->pressure < tp->pressure.low)
+				if (t->pressure < tp->pressure.low) {
+					evdev_log_debug(tp->device,
+							"pressure: end touch\n");
 					tp_end_touch(tp, t, time);
+				}
 			}
 		}
 
@@ -833,8 +916,7 @@ tp_unhover_pressure(struct tp_dispatch *tp, uint64_t time)
 	 * _all_ fingers have enough pressure, even if some of the slotted
 	 * ones don't. Anything else gets insane quickly.
 	 */
-	for (i = 0; i < (int)tp->ntouches; i++) {
-		t = tp_get_touch(tp, i);
+	tp_for_each_touch(tp, t) {
 		if (t->state == TOUCH_HOVERING) {
 			/* avoid jumps when landing a finger */
 			tp_motion_history_reset(t);
@@ -888,9 +970,7 @@ tp_unhover_fake_touches(struct tp_dispatch *tp, uint64_t time)
 	 */
 	if (tp_fake_finger_is_touching(tp) &&
 	    tp->nfingers_down < nfake_touches) {
-		for (i = 0; i < (int)tp->ntouches; i++) {
-			t = tp_get_touch(tp, i);
-
+		tp_for_each_touch(tp, t) {
 			if (t->state == TOUCH_HOVERING) {
 				tp_begin_touch(tp, t, time);
 
@@ -1048,7 +1128,6 @@ static void
 tp_process_state(struct tp_dispatch *tp, uint64_t time)
 {
 	struct tp_touch *t;
-	unsigned int i;
 	bool restart_filter = false;
 	bool want_motion_reset;
 
@@ -1058,9 +1137,7 @@ tp_process_state(struct tp_dispatch *tp, uint64_t time)
 
 	want_motion_reset = tp_need_motion_history_reset(tp);
 
-	for (i = 0; i < tp->ntouches; i++) {
-		t = tp_get_touch(tp, i);
-
+	tp_for_each_touch(tp, t) {
 		if (want_motion_reset) {
 			tp_motion_history_reset(t);
 			t->quirks.reset_motion_history = true;
@@ -1181,6 +1258,28 @@ tp_handle_state(struct tp_dispatch *tp,
 	tp_clickpad_middlebutton_apply_config(tp->device);
 }
 
+static inline void
+tp_debug_touch_state(struct tp_dispatch *tp,
+		     struct evdev_device *device)
+{
+	char buf[1024] = {0};
+	struct tp_touch *t;
+	size_t i = 0;
+
+	tp_for_each_touch(tp, t) {
+		if (i >= tp->nfingers_down)
+			break;
+		sprintf(&buf[strlen(buf)],
+			"slot %zd: %04d/%04d p%03d %s |",
+			i++,
+			t->point.x,
+			t->point.y,
+			t->pressure,
+			tp_touch_active(tp, t) ? "" : "inactive");
+	}
+	evdev_log_debug(device, "touch state: %s\n", buf);
+}
+
 static void
 tp_interface_process(struct evdev_dispatch *dispatch,
 		     struct evdev_device *device,
@@ -1204,6 +1303,9 @@ tp_interface_process(struct evdev_dispatch *dispatch,
 		break;
 	case EV_SYN:
 		tp_handle_state(tp, time);
+#if 0
+		tp_debug_touch_state(tp, device);
+#endif
 		break;
 	}
 }
@@ -1352,9 +1454,7 @@ tp_trackpoint_event(uint64_t time, struct libinput_event *event, void *data)
 		return;
 
 	if (!tp->palm.trackpoint_active) {
-		tp_edge_scroll_stop_events(tp, time);
-		tp_gesture_cancel(tp, time);
-		tp_tap_suspend(tp, time);
+		tp_stop_actions(tp, time);
 		tp->palm.trackpoint_active = true;
 	}
 
@@ -1467,9 +1567,7 @@ tp_keyboard_event(uint64_t time, struct libinput_event *event, void *data)
 				     ARRAY_LENGTH(tp->dwt.mod_mask)))
 		    return;
 
-		tp_edge_scroll_stop_events(tp, time);
-		tp_gesture_cancel(tp, time);
-		tp_tap_suspend(tp, time);
+		tp_stop_actions(tp, time);
 		tp->dwt.keyboard_active = true;
 		timeout = DEFAULT_KEYBOARD_ACTIVITY_TIMEOUT_1;
 	} else {
@@ -1486,8 +1584,6 @@ static bool
 tp_want_dwt(struct evdev_device *touchpad,
 	    struct evdev_device *keyboard)
 {
-	unsigned int bus_tp = libevdev_get_id_bustype(touchpad->evdev),
-		     bus_kbd = libevdev_get_id_bustype(keyboard->evdev);
 	unsigned int vendor_tp = evdev_device_get_id_vendor(touchpad);
 	unsigned int vendor_kbd = evdev_device_get_id_vendor(keyboard);
 	unsigned int product_tp = evdev_device_get_id_product(touchpad);
@@ -1497,23 +1593,12 @@ tp_want_dwt(struct evdev_device *touchpad,
 	   considered a happy couple */
 	if (touchpad->tags & EVDEV_TAG_EXTERNAL_TOUCHPAD)
 		return vendor_tp == vendor_kbd && product_tp == product_kbd;
+	else if (keyboard->tags & EVDEV_TAG_INTERNAL_KEYBOARD)
+		return true;
 
-	/* If the touchpad is on serio, the keyboard is too, so ignore any
-	   other devices */
-	if (bus_tp == BUS_I8042 && bus_kbd != bus_tp)
-		return false;
-
-	/* For Apple touchpads, always use its internal keyboard */
-	if (vendor_tp == VENDOR_ID_APPLE) {
-		return vendor_kbd == vendor_tp &&
-		       keyboard->model_flags &
-				EVDEV_MODEL_APPLE_INTERNAL_KEYBOARD;
-	}
-
-	/* everything else we don't really know, so we have to assume
-	   they go together */
-
-	return true;
+	/* keyboard is not tagged as internal keyboard and it's not part of
+	 * a combo */
+	return false;
 }
 
 static void
@@ -1521,7 +1606,9 @@ tp_dwt_pair_keyboard(struct evdev_device *touchpad,
 		     struct evdev_device *keyboard)
 {
 	struct tp_dispatch *tp = (struct tp_dispatch*)touchpad->dispatch;
-	unsigned int bus_kbd = libevdev_get_id_bustype(keyboard->evdev);
+
+	if (tp->dwt.keyboard)
+		return;
 
 	if ((keyboard->tags & EVDEV_TAG_KEYBOARD) == 0)
 		return;
@@ -1529,20 +1616,9 @@ tp_dwt_pair_keyboard(struct evdev_device *touchpad,
 	if (!tp_want_dwt(touchpad, keyboard))
 		return;
 
-	/* If we already have a keyboard paired, override it if the new one
-	 * is a serio device. Otherwise keep the current one */
-	if (tp->dwt.keyboard) {
-		if (bus_kbd != BUS_I8042)
-			return;
-
-		memset(tp->dwt.key_mask, 0, sizeof(tp->dwt.key_mask));
-		memset(tp->dwt.mod_mask, 0, sizeof(tp->dwt.mod_mask));
-		libinput_device_remove_event_listener(&tp->dwt.keyboard_listener);
-	}
-
 	libinput_device_add_event_listener(&keyboard->base,
-				&tp->dwt.keyboard_listener,
-				tp_keyboard_event, tp);
+					   &tp->dwt.keyboard_listener,
+					   tp_keyboard_event, tp);
 	tp->dwt.keyboard = keyboard;
 	tp->dwt.keyboard_active = false;
 
@@ -2187,21 +2263,14 @@ tp_init_dwt(struct tp_dispatch *tp,
 	return;
 }
 
-static void
-tp_init_palmdetect(struct tp_dispatch *tp,
-		   struct evdev_device *device)
+static inline void
+tp_init_palmdetect_edge(struct tp_dispatch *tp,
+			struct evdev_device *device)
 {
 	double width, height;
 	struct phys_coords mm = { 0.0, 0.0 };
 	struct device_coords edges;
 
-	tp->palm.right_edge = INT_MAX;
-	tp->palm.left_edge = INT_MIN;
-
-	if (device->tags & EVDEV_TAG_EXTERNAL_TOUCHPAD &&
-	    !tp_is_tpkb_combo_below(device))
-		return;
-
 	evdev_device_get_size(device, &width, &height);
 
 	/* Enable palm detection on touchpads >= 70 mm. Anything smaller
@@ -2217,8 +2286,28 @@ tp_init_palmdetect(struct tp_dispatch *tp,
 	mm.x = width * 0.95;
 	edges = evdev_device_mm_to_units(device, &mm);
 	tp->palm.right_edge = edges.x;
+}
+
+static void
+tp_init_palmdetect(struct tp_dispatch *tp,
+		   struct evdev_device *device)
+{
+
+	tp->palm.right_edge = INT_MAX;
+	tp->palm.left_edge = INT_MIN;
+
+	if (device->tags & EVDEV_TAG_EXTERNAL_TOUCHPAD &&
+	    !tp_is_tpkb_combo_below(device))
+		return;
 
 	tp->palm.monitor_trackpoint = true;
+
+	if (libevdev_has_event_code(device->evdev,
+				    EV_ABS,
+				    ABS_MT_TOOL_TYPE))
+		tp->palm.use_mt_tool = true;
+
+	tp_init_palmdetect_edge(tp, device);
 }
 
 static void
